package cn.ezhizao.project.business.service.impl;

import java.util.List;
import javax.annotation.Resource;

import cn.ezhizao.common.constant.UserConstants;
import cn.ezhizao.common.core.text.Convert;
import cn.ezhizao.common.exception.ServiceException;
import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.StringUtils;
import cn.ezhizao.common.utils.spring.SpringUtils;
import cn.ezhizao.framework.aspectj.lang.annotation.DataScope;
import cn.ezhizao.project.system.domain.SysUser;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import cn.ezhizao.project.business.mapper.SysDeptMapper;
import cn.ezhizao.project.business.domain.SysDept;
import cn.ezhizao.project.business.service.ISysDeptService;

/**
 * 部门Service业务层处理
 *
 * @author ezhizao
 * @date 2025-04-08
 */
@Service
public class SysDeptServiceImpl  extends ServiceImpl<SysDeptMapper, SysDept> implements ISysDeptService
{
    @Resource
    private SysDeptMapper sysDeptMapper;

    /**
     * 查询部门列表
     *
     * @param sysDept 部门
     * @return 部门
     */
    @Override
    public List<SysDept> getList(SysDept sysDept)
    {
        return sysDeptMapper.getList(sysDept);
    }
    @Override
    public String checkDeptNameUnique(SysDept dept)
    {
        Long deptId = StringUtils.isNull(dept.getDeptId()) ? -1L : dept.getDeptId();
        SysDept info = sysDeptMapper.checkDeptNameUnique(dept.getDeptName(), dept.getParentId());
        if (StringUtils.isNotNull(info) && info.getDeptId().longValue() != deptId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }
    @Override
    public String checkDeptCodeUnique(SysDept dept)
    {
        Long deptId = StringUtils.isNull(dept.getDeptId()) ? -1L : dept.getDeptId();
        SysDept info = sysDeptMapper.checkDeptCodeUnique(dept.getDeptCode(), dept.getParentId());
        if (StringUtils.isNotNull(info) && info.getDeptId().longValue() != deptId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }
    @Override
    public int insertDept(SysDept dept)
    {
        if(dept.getParentId() != 0) {
            SysDept info = sysDeptMapper.selectDeptById(dept.getParentId());
            if(info == null){
                throw new ServiceException("新增部门失败，上级部门不存在");
            }
            // 如果父节点不为正常状态,则不允许新增子节点
            if (!UserConstants.DEPT_NORMAL.equals(info.getStatus())) {
                throw new ServiceException("部门停用，不允许新增");
            }
            dept.setAncestors(info.getAncestors() + "," + dept.getParentId());
        }
        return sysDeptMapper.insertDept(dept);
    }
    @Override
    public int updateDept(SysDept dept)
    {
        SysDept newParentDept = sysDeptMapper.selectDeptById(dept.getParentId());
        SysDept oldDept = sysDeptMapper.selectDeptById(dept.getDeptId());
        if (StringUtils.isNotNull(newParentDept) && StringUtils.isNotNull(oldDept))
        {
            String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
            String oldAncestors = oldDept.getAncestors();
            dept.setAncestors(newAncestors);
            updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
        }
        int result = sysDeptMapper.updateDept(dept);
        if (UserConstants.DEPT_NORMAL.equals(dept.getStatus()) && StringUtils.isNotEmpty(dept.getAncestors())
                && !StringUtils.equals("0", dept.getAncestors()))
        {
            // 如果该部门是启用状态，则启用该部门的所有上级部门
            updateParentDeptStatusNormal(dept);
        }
        return result;
    }
    private void updateParentDeptStatusNormal(SysDept dept)
    {
        String ancestors = dept.getAncestors();
        Long[] deptIds = Convert.toLongArray(ancestors);
        sysDeptMapper.updateDeptStatusNormal(deptIds);
    }
    public void updateDeptChildren(Long deptId, String newAncestors, String oldAncestors)
    {
        List<SysDept> children = sysDeptMapper.selectChildrenDeptById(deptId);
        for (SysDept child : children)
        {
            child.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
        }
        if (children.size() > 0)
        {
            sysDeptMapper.updateDeptChildren(children);
        }
    }
    @Override
    public int selectNormalChildrenDeptById(Long deptId)
    {
        return sysDeptMapper.selectNormalChildrenDeptById(deptId);
    }
    @Override
    public int deleteDeptById(Long deptId)
    {
        return sysDeptMapper.deleteDeptById(deptId);
    }
    @Override
    public SysDept getDeptByDeptId(Long deptId) {
        return sysDeptMapper.getDeptByDeptId(deptId);
    }
    @Override
    public void checkDeptDataScope(Long deptId)
    {
        if (!SysUser.isAdmin(SecurityUtils.getUserId()))
        {
            SysDept dept = new SysDept();
            dept.setDeptId(deptId);
            List<SysDept> depts = SpringUtils.getAopProxy(this).selectDeptList(dept);
            if (StringUtils.isEmpty(depts))
            {
                throw new ServiceException("没有权限访问部门数据！");
            }
        }
    }
    @Override
    @DataScope(deptAlias = "d")
    public List<SysDept> selectDeptList(SysDept dept)
    {
        return sysDeptMapper.selectDeptList(dept);
    }
    @Override
    public boolean checkDeptExistUser(Long deptId)
    {
        int result = sysDeptMapper.checkDeptExistUser(deptId);
        return result > 0;
    }
    @Override
    public boolean hasChildByDeptId(Long deptId)
    {
        int result = sysDeptMapper.hasChildByDeptId(deptId);
        return result > 0;
    }
    /**
     * 物理删除
     * @param sysDept
     * @return 删除结果
     */
    @Override
    public int physicalDelete(SysDept sysDept){ return sysDeptMapper.physicalDelete(sysDept); };

}
